/* settings.cc
   Contains all the game settings

   Copyright (C) 2000  Mathias Broxvall

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "general.h"
#include "settings.h"
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <dirent.h>
using namespace std;

Settings *Settings::settings;
void Settings::init() {settings=new Settings();}
Settings::Settings() {
  mouseSensitivity=1.0;
  is_windowed=0; resolution = DEFAULT_RESOLUTION;
  gfx_details=3;
  showFPS=0;
  difficulty=0;
  sfxVolume=1.0; musicVolume=1.0;
  rotateArrows=0;

  /* Load all settings here */
  char str[256];
  snprintf(str,sizeof(str)-1,"%s/.trackballs/settings",getenv("HOME"));
  FILE *fp = fopen(str,"rb");
  if(fp) {
	int version;
	fread(&version,sizeof(int),1,fp);
	fread(&sfxVolume,sizeof(double),1,fp);
	fread(&musicVolume,sizeof(double),1,fp);
	fread(&mouseSensitivity,sizeof(double),1,fp);
	fread(&gfx_details,sizeof(int),1,fp);
	fread(&showFPS,sizeof(int),1,fp);
	fread(&is_windowed,sizeof(int),1,fp);
	fread(&resolution,sizeof(int),1,fp);

	/* TODO. Santitycheck of values */
	fclose(fp);
  }
}
void Settings::loadLevelSets() {
  /* Load all levelsets */
  char str[256];
  nLevelSets=0;

  /* ugly fix to make levelset lv.set the first level set */
  sprintf(str,"%s/levels/lv.set",SHARE_DIR);
  loadLevelSet(str);

  sprintf(str,"%s/levels",SHARE_DIR);
  DIR *dir = opendir(str);
  if(!dir) { printf("Can't find the %s/ directory\n",str); exit(0); }
  struct dirent *dirent;
  while((dirent = readdir(dir))) {
	if(strlen(dirent->d_name) > 4 && strcmp(&dirent->d_name[strlen(dirent->d_name)-4],".set") == 0) {
	  if(strcmp(dirent->d_name,"lv.set")) {
		sprintf(str,"%s/levels/%s",SHARE_DIR,dirent->d_name);
		loadLevelSet(str);
	  }
	}
  }

  snprintf(str,sizeof(str)-1,"%s/.trackballs/levels",getenv("HOME"));
  dir = opendir(str);
  if(dir)
	while((dirent = readdir(dir))) {
	  if(strlen(dirent->d_name) > 4 && strcmp(&dirent->d_name[strlen(dirent->d_name)-4],".set") == 0) {
		snprintf(str,sizeof(str)-1,"%s/.trackballs/levels/%s",getenv("HOME"),dirent->d_name);
		loadLevelSet(str);
	  }
	}


  if(!nLevelSets) {
	printf("Error: Failed to load any levelsets, place levels in %s/levels/",SHARE_DIR);
	exit(0);
  }
}
void Settings::loadLevelSet(char *str) {
  FILE *fp = fopen(str,"r");	  
  if(!fp)
	printf("Warning: Failure reading levelSet %s\n",str);
  else {
	fgets(levelSets[nLevelSets].name,256,fp);
	if(levelSets[nLevelSets].name[strlen(levelSets[nLevelSets].name)-1] == '\n')
	  levelSets[nLevelSets].name[strlen(levelSets[nLevelSets].name)-1]=0;
	fgets(levelSets[nLevelSets].startLevel,256,fp);
	if(levelSets[nLevelSets].startLevel[strlen(levelSets[nLevelSets].startLevel)-1] == '\n')
	  levelSets[nLevelSets].startLevel[strlen(levelSets[nLevelSets].startLevel)-1]=0;
	fgets(levelSets[nLevelSets].startLevelName,256,fp);
	if(levelSets[nLevelSets].startLevelName[strlen(levelSets[nLevelSets].startLevelName)-1] == '\n')
	  levelSets[nLevelSets].startLevelName[strlen(levelSets[nLevelSets].startLevelName)-1]=0;
	nLevelSets++;
	fclose(fp);
  }
}


void Settings::save() {
  char str[256];
  int version=1;

  snprintf(str,sizeof(str)-1,"%s/.trackballs",getenv("HOME"));

  mkdir(str,S_IXUSR|S_IRUSR|S_IWUSR|S_IXGRP|S_IRGRP|S_IWGRP);
  snprintf(str,sizeof(str)-1,"%s/.trackballs/settings",getenv("HOME"));
  
  /* TODO. Save all settings here */
  FILE *fp = fopen(str,"wb");
  if(!fp) { printf("Warning. Could not save settins\n"); }
  else {
	fwrite(&version,sizeof(int),1,fp);
	fwrite(&sfxVolume,sizeof(double),1,fp);
	fwrite(&musicVolume,sizeof(double),1,fp);
	fwrite(&mouseSensitivity,sizeof(double),1,fp);
	fwrite(&gfx_details,sizeof(int),1,fp);
	fwrite(&showFPS,sizeof(int),1,fp);
	fwrite(&is_windowed,sizeof(int),1,fp);
	fwrite(&resolution,sizeof(int),1,fp);
	fclose(fp);
  }
}
